@@ -1,6 +1,6 @@
 /*
  * GAI - General Applet Interface Library
- * Copyright (C) 2003-2004 Jonas Aaberg <cja@gmx.net>
+ * Copyright (C) 2003-2005 Jonas Aaberg <cja@gmx.net>
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -269,10 +269,10 @@ gai_dockapp_window_windowmaker(void)
 
 #ifdef GAI_WITH_GL
     if(GAI.open_gl)
-	gai_display_error_quit("You're trying to run an applet that uses OpenGL.\n"
+	gai_display_error_quit(_("You're trying to run an applet that uses OpenGL.\n"
 			       "gtkglext doesn't work together with WindowMaker\n"
 			       "special way of docking applets.\n"
-			       "I'm sorry.\n");
+			       "I'm sorry.\n"));
     /*
     if(GAI.open_gl){
 	gtk_widget_set_gl_capability(GAI.drawingarea,
@@ -381,16 +381,92 @@ gai_timer(gpointer d)
     return TRUE;
 }
 
+static void gai_create_menu(void)
+{
+    GtkWidget *menu_line=NULL;
+    GtkAccelGroup *accel_group;
+    GdkPixbuf *pixbuf;
+    MenuHashItem *hash_item;
+    gint i;
+
+    GAI_ENTER;
+
+    if(!GAI.menu_changed)
+	return;
+
+    GAI.menu_changed = FALSE;
+
+    accel_group = gtk_accel_group_new();
+
+    if(GAI.menu != NULL){
+	gtk_widget_destroy(GAI.menu);
+	//g_object_unref(GAI.menu);
+    }
+
+    GAI.menu = gtk_menu_new();
+
+    for(i=0;i<g_slist_length(GAI.menu_list);i++){
+
+	hash_item = (MenuHashItem *)g_hash_table_lookup(GAI.menu_hash, g_slist_nth_data(GAI.menu_list, i));
+
+	if(hash_item->type == GAI_MENU_STOCK || hash_item->type == GAI_MENU_NONE || hash_item->type == GAI_MENU_FILE){
+
+	    menu_line = gtk_image_menu_item_new_with_mnemonic(hash_item->name);
+
+
+	    if(hash_item->icon != NULL){
+
+		if(hash_item->type == GAI_MENU_STOCK){
+
+		    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menu_line), 
+						   gtk_image_new_from_stock(hash_item->icon,
+									     GTK_ICON_SIZE_MENU));
+		}
+
+		if(hash_item->type == GAI_MENU_FILE) {
+		    pixbuf = gai_load_image(hash_item->icon);
+
+		    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menu_line), 
+						   gtk_image_new_from_pixbuf(pixbuf));
+		    
+		    g_object_unref(pixbuf);
+		}
+	    }
+
+
+	    if(hash_item->name != NULL)
+		g_signal_connect(G_OBJECT(menu_line), "activate",
+				 G_CALLBACK(hash_item->func),
+				 hash_item->ptr);
+
+	}
+	if(hash_item->type == GAI_MENU_SEPARATOR){
+	    menu_line = gtk_menu_item_new();
+	    gtk_widget_set_sensitive(menu_line, FALSE);
+	}
+
+	gtk_container_add(GTK_CONTAINER(GAI.menu), menu_line);
+    }
+    
+    gtk_menu_set_accel_group(GTK_MENU(GAI.menu), accel_group);
+
+    gtk_widget_show_all(GAI.menu);
+
+    GAI_LEAVE;
+}
+
+
 static int 
 on_mouse_click_callback(GtkWidget *widget, GdkEventButton *e, gpointer d)
 {
     int x, y;
     GAI_ENTER;
 
-    if(e->button == 3)
+    if(e->button == 3 && e->type == GDK_BUTTON_PRESS)
     {
 	if(GAI.applet_type == GAI_DOCKAPP || GAI.applet_type == GAI_ROX)
 	{
+	    gai_create_menu();
 	    gtk_menu_popup(GTK_MENU(GAI.menu), NULL, NULL, NULL, NULL, 
 			    e->button, e->time);
 	    return TRUE;
@@ -399,17 +475,6 @@ on_mouse_click_callback(GtkWidget *widget, GdkEventButton *e, gpointer d)
 	return FALSE;
     }
 
-    /*
-    if(GAI.rotate && GAI.orient == GAI_VERTICAL)
-    {
-	x=(int)(e->y/GAI.scale);
-	y=(int)(e->x/GAI.scale);
-
-    }else{
-	x=(int)(e->x/GAI.scale);
-	y=(int)(e->y/GAI.scale);
-    }*/
-
 
     x=(int)(e->x/GAI.scale);
     y=(int)(e->y/GAI.scale);
@@ -417,7 +482,7 @@ on_mouse_click_callback(GtkWidget *widget, GdkEventButton *e, gpointer d)
 
     GAI_D("click:(%d, %d) scaled to(%d, %d)\n",(int)e->x, (int)e->y, x,y);
 
-    if(e->button==1)
+    if(e->button==1 && e->type == GDK_BUTTON_PRESS)
     {  
 	if(GAI.on_mouse_click1_callback)
 	    GAI.on_mouse_click1_callback(x,y, 
@@ -426,7 +491,7 @@ on_mouse_click_callback(GtkWidget *widget, GdkEventButton *e, gpointer d)
 	return TRUE;
     }
 
-    if (e->button==2)
+    if (e->button==2 && e->type == GDK_BUTTON_PRESS)
     {  
 	if (GAI.on_mouse_click2_callback)
 	    GAI.on_mouse_click2_callback(x,y, 
@@ -759,6 +824,9 @@ static void gai_simple_about(void)
     g_signal_connect((gpointer)about_ok_button, "clicked",
 		      G_CALLBACK(gai_simple_about_ok), NULL);
 
+    g_signal_connect((gpointer)about_ok_button, "destroy",
+		      G_CALLBACK(gai_simple_about_ok), NULL);
+
     gtk_widget_show_all(GAI.about);
     GAI_LEAVE;
 
@@ -792,7 +860,7 @@ gai_on_preferences_activate(GtkMenuItem *menuitem, gpointer d)
     if(GAI.has_preferences == GAI_PREF_OWN)
     {
 	if(GAI.on_preferences_callback)
-	    GAI.on_preferences_callback(GAI.on_preferences_userdata);
+	    GAI.on_preferences_callback(FALSE, GAI.on_preferences_userdata);
     }
     else if(GAI.has_preferences == GAI_PREF_GEN)
     {
@@ -821,6 +889,7 @@ void
 gai_on_about_activate(GtkMenuItem *menuitem, gpointer d)
 {
     GAI_ENTER;
+
 #ifdef GAI_WITH_GNOME
     if(GAI.applet_type == GAI_GNOME1 ||
 	GAI.applet_type == GAI_GNOME2)
@@ -852,78 +921,6 @@ gai_on_remove_activate(GtkMenuItem *menuitem, gpointer d)
 
 
 
-static void 
-gai_create_menu(void)
-{
-    GtkWidget *menu_line=NULL;
-    GtkAccelGroup *accel_group;
-    GdkPixbuf *pixbuf;
-    int i;
-
-    GAI_ENTER;
-
-    accel_group = gtk_accel_group_new();
-    GAI.menu = gtk_menu_new();
-
-    for(i=0;i<GAI.menu_entries;i++){
-
-	if(g_array_index(GAI.menu_type,int,i) == GAI_MENU_STOCK ||
-	   g_array_index(GAI.menu_type,int,i) == GAI_MENU_NONE ||
-	   g_array_index(GAI.menu_type,int,i) == GAI_MENU_FILE){
-
-	    menu_line = gtk_image_menu_item_new_with_mnemonic(g_ptr_array_index(GAI.menu_name,i));
-	    /* Freeing up allocated name, no need to keep it. */
-	    g_free(g_ptr_array_index(GAI.menu_name,i));
-
-	    if(g_ptr_array_index(GAI.menu_icon,i) != NULL){
-
-		if(g_array_index(GAI.menu_type,int,i) == GAI_MENU_STOCK){
-
-		    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menu_line), 
-						   gtk_image_new_from_stock(g_ptr_array_index(GAI.menu_icon,i),
-									     GTK_ICON_SIZE_MENU));
-		}
-
-		if(g_array_index(GAI.menu_type,int,i) == GAI_MENU_FILE) {
-		    pixbuf = gai_load_image(g_ptr_array_index(GAI.menu_icon,i));
-
-		    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menu_line), 
-						   gtk_image_new_from_pixbuf(pixbuf));
-		    
-		    g_object_unref(pixbuf);
-		}
-		/* Freeing up icon */
-		g_free(g_ptr_array_index(GAI.menu_icon,i));
-	    }
-
-
-	    if(g_ptr_array_index(GAI.menu_func,i) !=NULL)
-		g_signal_connect(G_OBJECT(menu_line), "activate",
-				  G_CALLBACK(g_ptr_array_index(GAI.menu_func,i)),
-				  g_ptr_array_index(GAI.menu_ptr,i));
-
-	}
-	if(g_array_index(GAI.menu_type,int,i) == GAI_MENU_SEPARATOR){
-	    menu_line = gtk_menu_item_new();
-	    gtk_widget_set_sensitive(menu_line, FALSE);
-	}
-
-	gtk_container_add(GTK_CONTAINER(GAI.menu), menu_line);
-    }
-    
-    gtk_menu_set_accel_group(GTK_MENU(GAI.menu), accel_group);
-    
-    gtk_widget_show_all(GAI.menu);
-
-    /* Destroy the arrays, they are of no use now */
-    g_array_free(GAI.menu_type, TRUE);
-    g_ptr_array_free(GAI.menu_name,TRUE);
-    g_ptr_array_free(GAI.menu_icon,TRUE);
-    g_ptr_array_free(GAI.menu_func,TRUE);
-    g_ptr_array_free(GAI.menu_ptr,TRUE);
-
-    GAI_LEAVE;
-}
 
 
 void gai_dockapp_main(void)
@@ -955,7 +952,7 @@ void gai_dockapp_main(void)
     }
 
     gai_hook();
-    gai_create_menu();
+    //    gai_create_menu();
 
 
 #ifdef GAI_WITH_GL
