@@ -275,10 +275,10 @@ gai_background_set(int width, int height, int max_size, int border)
 
     GAI_D("width: %d height: %d\n",width, height);
 
-    g_assert((width >0) && (width < 1000));
-    g_assert((height >0) && (height < 1000));
+    g_assert((width >0) && (width < 1280*2));
+    g_assert((height >0) && (height < 1280*2));
 
-    g_assert((max_size >=GAI_BACKGROUND_MAX_SIZE_IMAGE) && (max_size<1000));
+    g_assert((max_size >=GAI_BACKGROUND_MAX_SIZE_IMAGE) && (max_size < 1280*2));
     g_assert((border == TRUE) || (border == FALSE));
 
     GAI.use_default_background = 1;
@@ -318,7 +318,7 @@ gai_background_from_gdkpixbuf(GdkPixbuf *pixbuf, int max_size)
 
 
     g_assert(pixbuf != NULL);
-    g_assert((max_size >= GAI_BACKGROUND_MAX_SIZE_IMAGE) && (max_size < 3000));
+    g_assert((max_size >= GAI_BACKGROUND_MAX_SIZE_IMAGE) && (max_size < 1280*2));
 
 
     GAI.use_default_background = 0;
@@ -352,7 +352,7 @@ gai_background_from_gdkpixbuf(GdkPixbuf *pixbuf, int max_size)
 	e.width = gdk_pixbuf_get_width(pixbuf);
 	e.height = gdk_pixbuf_get_height(pixbuf);
 	gdk_window_get_position(GAI.widget->window, &e.x, &e.y);
-	printf("root_win, x=%d y=%d w=%d h=%d\n", e.x, e.y, e.width, e.height); 
+	//printf("root_win, x=%d y=%d w=%d h=%d\n", e.x, e.y, e.width, e.height); 
 	gai_root_window_config(NULL, &e, (gpointer) -1);
     }
 
@@ -389,8 +389,8 @@ gai_background_from_file(const char *file, int max_size)
 
     if(GAI.applet.image_path == NULL)
     {
-	gai_display_error_quit("No image_path is set!\n"
-			       "That is required before loading images.\n");
+	gai_display_error_quit(_("No image_path is set!\n"
+			       "That is required before loading images.\n"));
 	return;
     }
 
