@@ -41,10 +41,6 @@
 #define OAF2_SERVER  "GAI-%s-Applet"
 #define OAF2_FACTORY "GAI-%s-Applet-Factory"
 
-/* Gnome makes a ugly border of a pixel around the applet */
-#define GNOME_STEALS 2
-
-
 
 int
 gai_gnome_detect_applet_type (int argc, char** argv)
@@ -57,10 +53,10 @@ gai_gnome_detect_applet_type (int argc, char** argv)
 	!strncmp(argv[2], "--oaf-ior-fd=",13))
     {
 #      ifndef GAI_WITH_GNOME
-	gai_display_error_quit("You're trying to run this applet"
+	gai_display_error_quit(_("You're trying to run this applet"
 			       " as a Gnome panel applet\n"
 			       "and there is no Gnome support"
-			       " compiled in for the GAI library.\n");
+			       " compiled in for the GAI library.\n"));
 #      endif
 	return GAI_GNOME2;
     }
@@ -70,13 +66,13 @@ gai_gnome_detect_applet_type (int argc, char** argv)
 	!strncmp(argv[2], "--oaf-ior-fd=",13))
     {
 #      ifndef GAI_WITH_GNOME
-	gai_display_error_quit("You're trying to run this applet"
+	gai_display_error_quit(_("You're trying to run this applet"
 			       " as a Gnome panel applet\n"
 			       "and there is no Gnome support"
-			       " compiled in for the GAI library.\n");
+			       " compiled in for the GAI library.\n"));
 #      endif
 	return GAI_GNOME1;
-    }else{
+    } else {
 	return GAI_DOCKAPP; 
     }
 }
@@ -174,68 +170,80 @@ void gai_size_change(int default_size, int curr_w, int curr_h, int gai_size_init
 
 #ifdef GAI_WITH_GNOME
 
-
-static BonoboUIVerb *gnome_menu_verb;
-static char *gnome_menu_xml;
-
-static void gai_gnome_create_menu(void)
+void gai_gnome_create_menu(void)
 {
-    int i,ptr,j=0;
-    gnome_menu_verb = g_malloc0(sizeof(BonoboUIVerb)*GAI.menu_entries);
+    int i, j=0;
+    MenuHashItem *item;
+    char *tmp = NULL;
 
-    /* It will be freed anyway pretty soon */
-    gnome_menu_xml = g_malloc0(81920);
+    if(GAI.menu_verb != NULL)
+	g_free(GAI.menu_verb);
 
-    sprintf(gnome_menu_xml, "<popup name=\"button3\">\n");
-    ptr = strlen(gnome_menu_xml);
+    if(GAI.menu_xml != NULL)
+	g_free(GAI.menu_xml);
 
-    GAI_D("Number of gai entries:%d",GAI.menu_entries);
+    GAI.menu_verb = g_malloc0(sizeof(BonoboUIVerb) * g_slist_length(GAI.menu_list)+1);
 
-    for(i=0;i<GAI.menu_entries;i++){
-	if(g_array_index(GAI.menu_type, int, i)  == GAI_MENU_STOCK ||
-	    g_array_index(GAI.menu_type, int, i)  == GAI_MENU_NONE ||
-	    g_array_index(GAI.menu_type, int, i)  == GAI_MENU_FILE ){
+    GAI.menu_xml = g_strdup("<popup name=\"button3\">\n");
 
-	    gnome_menu_verb[j].cname = g_strdup_printf("%.4d",i);
-	    gnome_menu_verb[j].cb = (BonoboUIVerbFn)g_ptr_array_index(GAI.menu_func,i);
 
-	    if(g_array_index(GAI.menu_type, int, i)  == GAI_MENU_STOCK)
-		sprintf(gnome_menu_xml+ptr, "<menuitem name=\"%.4d\" verb=\"%.4d\" \n"
-			"_label=\"%s\" pixtype=\"stock\" pixname=\"%s\"/>\n",i,i,(char *)g_ptr_array_index(GAI.menu_name,i),
-			(char *)g_ptr_array_index(GAI.menu_icon,i));
-	    if(g_array_index(GAI.menu_type, int, i)  == GAI_MENU_NONE)
-		sprintf(gnome_menu_xml+ptr, "<menuitem name=\"%.4d\" verb=\"%.4d\" \n"
-			"_label=\"%s\"/>\n",i,i,(char *)g_ptr_array_index(GAI.menu_name,i));
-	    if(g_array_index(GAI.menu_type, int, i)  == GAI_MENU_FILE){
+    GAI_D("Number of gai entries:%d",g_slist_length(GAI.menu_list));
 
-		sprintf(gnome_menu_xml+ptr, "<menuitem name=\"%.4d\" verb=\"%.4d\" \n"
-			"_label=\"%s\" pixtype=\"filename\" pixname=\"%s/%s\"/>\n",i,i,
-			(char *)g_ptr_array_index(GAI.menu_name,i),
-			GAI.applet.image_path,
-			(char *)g_ptr_array_index(GAI.menu_icon,i));
+    for(i=0;i<g_slist_length(GAI.menu_list);i++){
 
+	item = g_hash_table_lookup(GAI.menu_hash, g_slist_nth_data(GAI.menu_list, i));
+
+	if(item->type  == GAI_MENU_STOCK || item->type  == GAI_MENU_NONE || item->type  == GAI_MENU_FILE ){
+
+	    GAI.menu_verb[j].cname = g_strdup_printf("%.4d",i);
+	    GAI.menu_verb[j].cb = (BonoboUIVerbFn)item->func;
+	    GAI.menu_verb[j].user_data = NULL;
+
+
+	    if(item->type  == GAI_MENU_STOCK){
+
+		tmp = g_strdup_printf("%s<menuitem name=\"%.4d\" verb=\"%.4d\" \n"
+			"_label=\"%s\" pixtype=\"stock\" pixname=\"%s\"/>\n",GAI.menu_xml,i,i,item->name, item->icon);
 	    }
 
+	    if(item->type  == GAI_MENU_NONE){
+		tmp = g_strdup_printf("%s<menuitem name=\"%.4d\" verb=\"%.4d\" \n"
+			"_label=\"%s\"/>\n", GAI.menu_xml, i, i, item->name);
+	    }
+
+	    if(item->type  == GAI_MENU_FILE){
+
+		tmp = g_strdup_printf("%s<menuitem name=\"%.4d\" verb=\"%.4d\" \n"
+				      "_label=\"%s\" pixtype=\"filename\" pixname=\"%s/%s\"/>\n", GAI.menu_xml, i, i,
+				      item->name, GAI.applet.image_path, item->icon);
+
+	    }
+	    g_free(GAI.menu_xml);
+	    GAI.menu_xml = tmp;
 	    j++;
 	}
 
-	/* Seems to be buggy in a way */
-	if(g_array_index(GAI.menu_type, int, i) == GAI_MENU_SEPARATOR)
-	    sprintf(gnome_menu_xml+ptr,"<separator/>\n");
-	ptr = strlen(gnome_menu_xml);
+	if(item->type == GAI_MENU_SEPARATOR){
+	    tmp = g_strdup_printf("%s<separator/>\n", GAI.menu_xml);
+	    g_free(GAI.menu_xml);
+	    GAI.menu_xml = tmp;
+	}
+
 
     }
-    sprintf(gnome_menu_xml+ptr,"</popup>\n");
+
+    tmp = g_strdup_printf("%s</popup>\n", GAI.menu_xml);
+    g_free(GAI.menu_xml);
+    GAI.menu_xml = tmp;
+
+    GAI_NOTE(GAI.menu_xml);
+    GAI.menu_changed = FALSE;
 
 
-    /* Destroy the arrays, they are of no use now */
-    g_array_free(GAI.menu_type, TRUE);
-    g_ptr_array_free(GAI.menu_name,TRUE);
-    g_ptr_array_free(GAI.menu_icon,TRUE);
-    g_ptr_array_free(GAI.menu_func,TRUE);
-    g_ptr_array_free(GAI.menu_ptr,TRUE);
 
-    GAI_NOTE(gnome_menu_xml);
+    panel_applet_setup_menu(PANEL_APPLET(GAI.widget), 
+			    GAI.menu_xml, 
+			    GAI.menu_verb, NULL);
 
 }
 
@@ -420,10 +428,10 @@ gai_gnome_start(PanelApplet *widget)
 
 #ifdef GAI_WITH_GL
     if(GAI.open_gl)
-	gai_display_error_quit("You're trying to run an applet that uses OpenGL.\n"
+	gai_display_error_quit(_("You're trying to run an applet that uses OpenGL.\n"
 			       "Open GL doesn't work on the Gnome Panel. Or it is\n"
 			       "a bug in the GAI library. Which ever it is,\n"
-			       "I'm sorry..\n");
+			       "I'm sorry..\n"));
 
 /*    if(GAI.open_gl){
 
@@ -500,9 +508,6 @@ gai_gnome_start(PanelApplet *widget)
 
     gai_gnome_create_menu();
 
-    panel_applet_setup_menu(PANEL_APPLET(GAI.widget), 
-			    gnome_menu_xml, 
-			    gnome_menu_verb, NULL);
 
     if(GAI.orient == GAI_HORIZONTAL){
 	GAI_D("default_height:%d, panel:%d\n",GAI.default_height, panel_applet_get_size(PANEL_APPLET(GAI.widget)));
@@ -550,10 +555,9 @@ gai_gnome_start(PanelApplet *widget)
 
     gtk_widget_show_all(GAI.widget);
 
-    /* Stupid ? */
 
-    g_free(gnome_menu_xml);
-    g_free(gnome_menu_verb);
+//    g_free(gnome_menu_xml);
+//    g_free(gnome_menu_verb);
 
 
     GAI.gnome_started = 1;
@@ -596,8 +600,8 @@ gai_gnome_factory (PanelApplet *applet, const gchar *id, gpointer d)
 	    gnome_instance_running = 1;
 	}
 	else {
-	    gai_display_error_continue("Sorry, it's not possible to run\n"
-		                       "two instances of the same applet.\n");
+	    gai_display_error_continue(_("Sorry, it's not possible to run\n"
+		                       "two instances of the same applet.\n"));
 	    return TRUE;
 	}
     }else{
@@ -698,7 +702,7 @@ void gai_gnome_about_show (void)
     }
 
     authors[0] = GAI.applet.author;
-    authors[1] = g_strdup("\nThis applet uses the GAI library\n - http://gai.sourceforge.net - \n");
+    authors[1] = g_strdup(_("\nThis applet uses the GAI library\n - http://gai.sourceforge.net - \n"));
     authors[2] = NULL;
 
     pixbuf = gdk_pixbuf_new_from_file(GAI.applet.icon, NULL);
