@@ -113,14 +113,14 @@ on_close_button_clicked(GtkButton *b, gpointer d)
 static void 
 on_apply_button_clicked(GtkButton *b, gpointer d)
 {
-    GAI_NOTE("apply");
+    GAI_NOTE(_("apply"));
     gai_pref_get_answers();
     if (GAI.on_preferences_callback)
     {
 	GAI.restarting=1;
 
 	if (GAI.on_preferences_callback)
-	    GAI.on_preferences_callback(GAI.on_preferences_userdata);
+	    GAI.on_preferences_callback(FALSE, GAI.on_preferences_userdata);
 
 	GAI.restarting=0;
     }
@@ -371,10 +371,10 @@ create_main_buttons(void)
 
     buttonbar = gtk_hbox_new (FALSE, 0);
 
-    ok_button =     make_pic_button(" Ok  ",GTK_STOCK_OK,    BUTTON_PIC_LABEL);
-    apply_button =  make_pic_button("Apply",GTK_STOCK_APPLY, BUTTON_PIC_LABEL);
-    close_button =  make_pic_button("Close",GTK_STOCK_CLOSE, BUTTON_PIC_LABEL);
-    help_button =   make_pic_button("Help ",GTK_STOCK_HELP,  BUTTON_PIC_LABEL);
+    ok_button =     make_pic_button(_(" Ok  "),GTK_STOCK_OK,    BUTTON_PIC_LABEL);
+    apply_button =  make_pic_button(_("Apply"),GTK_STOCK_APPLY, BUTTON_PIC_LABEL);
+    close_button =  make_pic_button(_("Close"),GTK_STOCK_CLOSE, BUTTON_PIC_LABEL);
+    help_button =   make_pic_button(_("Help "),GTK_STOCK_HELP,  BUTTON_PIC_LABEL);
     
     gtk_box_pack_start (GTK_BOX (buttonbar), ok_button,    FALSE, FALSE, 0);
     gtk_box_pack_start (GTK_BOX (buttonbar), apply_button, FALSE, FALSE, 0);
@@ -967,8 +967,8 @@ gai_pref_get_answers(void)
 
 	    if((unsigned char*)(((int *)iw[i].result)[0]) != NULL &&
 	       (unsigned char*)(((int *)iw[i].default_val)[0]) != NULL)
-		(unsigned char*)(((int *)iw[i].result)[0]) = 
-		    (unsigned char*)(((int *)iw[i].default_val)[0]) = g_strdup(buff);
+		(((int *)iw[i].result)[0]) = 
+		    (((int *)iw[i].default_val)[0]) = g_strdup(buff);
 
 
 	    //printf("%s\n",buff);
@@ -988,7 +988,7 @@ gai_pref_get_answers(void)
 
 	    
 	    if(j == g_list_length((GList *)((int *)iw[i].result)[0])){
-		(GList *)((int *)iw[i].result)[0] = g_list_append((GList *)((int *)iw[i].result)[0], 
+		((int *)iw[i].result)[0] = g_list_append((GList *)((int *)iw[i].result)[0], 
 								  g_strdup(buff));
 		//		printf("%s is a new entry\n",buff);
 	    } 
@@ -1025,7 +1025,7 @@ gai_pref_get_answers(void)
 
 		//printf("j=%d -  %d %d %d\n",j,cs[j].r,cs[j].g,cs[j].b);
 	    } else {
-		printf("GAI: Didn't find colour selection!\n");
+		printf(_("GAI: Didn't find colour selection!\n"));
 	    }
 	}
 	else if(iw[i].type == GAI_OPTIONMENU)
