@@ -56,7 +56,7 @@ GAI_struct* gai_instance = NULL;
 void 
 gai_is_init (void)
 {
-    char *msg = "First function must be gai_init()!";
+    char *msg = _("First function must be gai_init()!");
 
     if (gai_instance == NULL)
     {
@@ -88,6 +88,9 @@ GdkGC *
 gai_get_gc(void)
 {
     GAI_CHECK;  gai_is_init();
+    if(GAI.gc == NULL)
+	GAI.gc = gdk_gc_new(GAI.window);
+
     return GAI.gc;
 }
 
@@ -162,10 +165,10 @@ void gai_gl_init_func(GaiCallback0 function)
     GAI.gl_init_func = function;
     GAI.open_gl = 1;
 #else
-    gai_display_error_quit("You're trying to run an applet that uses OpenGL.\n"
+    gai_display_error_quit(_("You're trying to run an applet that uses OpenGL.\n"
 			   "You have not compiled with OpenGL support in GAI.\n"
 			   "Please visit http://gtkglext.sf.net for GtkGlExt\n"
-			   "which is required for OpenGL support.\n");
+			   "which is required for OpenGL support.\n"));
 #endif
     GAI_LEAVE;
 }
@@ -252,23 +255,23 @@ gai_start(void)
 #endif
 
     if(GAI.has_preferences != GAI_PREF_NONE)
-	gai_menu_add("Preferences...", "gtk-properties", GAI_MENU_STOCK, gai_on_preferences_activate, NULL);
+	gai_menu_add(_("Preferences..."), "gtk-properties", GAI_MENU_STOCK, gai_on_preferences_activate, NULL);
     if(GAI.use_help)
-	gai_menu_add("Help","gtk-help", GAI_MENU_STOCK, gai_on_help_activate, NULL);
+	gai_menu_add(_("Help"),"gtk-help", GAI_MENU_STOCK, gai_on_help_activate, NULL);
 
     switch (GAI.applet_type)
     {
     case GAI_DOCKAPP:
-	gai_menu_add("About...","gtk-yes", GAI_MENU_STOCK, gai_on_about_activate, NULL);
+	gai_menu_add(_("About..."),"gtk-yes", GAI_MENU_STOCK, gai_on_about_activate, NULL);
 	gai_menu_add(NULL, NULL, GAI_MENU_SEPARATOR, NULL, NULL);
-	gai_menu_add("Remove From Dock", "gtk-remove", GAI_MENU_STOCK, gai_on_remove_activate, NULL);
+	gai_menu_add(_("Remove From Dock"), "gtk-remove", GAI_MENU_STOCK, gai_on_remove_activate, NULL);
 	GAI.init_done = 1;
 	gai_dockapp_main();
 	break;
 #  ifdef GAI_WITH_GNOME
     case GAI_GNOME1:
     case GAI_GNOME2:
-	gai_menu_add("About...","gnome-stock-about", GAI_MENU_STOCK, gai_on_about_activate, NULL);
+	gai_menu_add(_("About..."),"gnome-stock-about", GAI_MENU_STOCK, gai_on_about_activate, NULL);
 	GAI.init_done = 1;
 	gai_gnome_main();
 	break;
@@ -276,18 +279,18 @@ gai_start(void)
 
 #  ifdef GAI_WITH_ROX
     case GAI_ROX:
-	gai_menu_add("About...","gtk-yes", GAI_MENU_STOCK, gai_on_about_activate, NULL);
+	gai_menu_add(_("About..."),"gtk-yes", GAI_MENU_STOCK, gai_on_about_activate, NULL);
 	gai_menu_add(NULL, NULL, GAI_MENU_SEPARATOR, NULL, NULL);
-	gai_menu_add("Remove From Rox Panel", "gtk-remove", GAI_MENU_STOCK, gai_on_remove_activate, NULL);
+	gai_menu_add(_("Remove from Rox Panel"), "gtk-remove", GAI_MENU_STOCK, gai_on_remove_activate, NULL);
 	GAI.init_done = 1;
 	gai_dockapp_main();
 	break;
 #  endif
 #  ifdef GAI_WITH_KDE
     case GAI_KDE:
-	gai_menu_add("About...","gtk-yes", GAI_MENU_STOCK, gai_on_about_activate, NULL);
+	gai_menu_add(_("About..."),"gtk-yes", GAI_MENU_STOCK, gai_on_about_activate, NULL);
 	gai_menu_add(NULL, NULL, GAI_MENU_SEPARATOR, NULL, NULL);
-	gai_menu_add("Remove From Rox Panel", "gtk-remove", GAI_MENU_STOCK, gai_on_remove_activate, NULL);
+	gai_menu_add(_("Remove from KDE Panel"), "gtk-remove", GAI_MENU_STOCK, gai_on_remove_activate, NULL);
 	GAI.init_done = 1;
 	gai_dockapp_main();
 
@@ -302,6 +305,19 @@ gai_start(void)
 }
 
 
+static void menu_hash_free_key(int *key)
+{
+    g_free(key);
+}
+
+static void menu_hash_free_data(MenuHashItem *item)
+{
+    if(item->name != NULL)
+	g_free(item->name);
+    if(item->icon != NULL)
+	g_free(item->icon);
+    g_free(item);
+}
 
 
 
@@ -310,13 +326,10 @@ gai_init_instance (GaiApplet *applet, const char *name, const char *version,  co
 		   int *argc, char ***argv)
 {
     char *gnome_dir = NULL;
-
-
-    /* Should generate an error message actually */
-/*    if(gai_instance)
-	return -1;
-*/
-
+#ifdef ENABLE_NLS
+    gint i, count;
+    gchar *locale_start;
+#endif
 
     /* Make sure ~/.gnome2 exists */
     gnome_dir = g_strdup_printf("%s/.gnome2",getenv("HOME"));
@@ -324,7 +337,53 @@ gai_init_instance (GaiApplet *applet, const char *name, const char *version,  co
     mkdir(gnome_dir, S_IRWXU);
     g_free(gnome_dir);
 
-    gai_instance = g_new0 (GAI_struct,1);
+    gai_instance = g_new0(GAI_struct, 1);
+
+
+
+#ifdef ENABLE_NLS
+    /* For GAI itself. */
+    bindtextdomain(GETTEXT_PACKAGE, GAILOCALEDIR);
+    bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+    textdomain(GETTEXT_PACKAGE);
+
+    /* For the applet - Only for gai_init2 */
+    if(applet != NULL){
+
+	/* until next version of the API, use image path minus last two dirs */
+
+	count = 0;
+	for(i = strlen(applet->image_path)-1; i > 0 ;i--){
+	    if(applet->image_path[i] == '/'){
+		count++;
+		if(count == 2)
+		    break;
+	    }
+	}
+	locale_start = g_malloc0(i+1);
+	memcpy(locale_start, applet->image_path, i);
+	GAI.locale = g_strdup_printf("%s/locale/", locale_start);
+	g_free(locale_start);
+
+
+	/* 
+	   
+	dcgettext(applet->name, msg, locale);
+	*/
+
+	/* Set it! */
+	bindtextdomain(applet->name, GAI.locale);
+	bind_textdomain_codeset(applet->name, "UTF-8");
+	/*textdomain(applet->name);*/
+
+    }
+
+
+
+
+#endif
+
+
 
 
 #ifdef GAI_WITH_GL
@@ -386,7 +445,7 @@ gai_init_instance (GaiApplet *applet, const char *name, const char *version,  co
     GAI.background = NULL;
     GAI.orig_background = NULL;
     GAI.draw_bg_update_done = 0;
-
+    GAI.gc = NULL;
 
     /* Start-up with no icon window */
     GAI.icon_window = NULL;
@@ -427,13 +486,14 @@ gai_init_instance (GaiApplet *applet, const char *name, const char *version,  co
 
     GAI.parent_window = -1;
 
+    GAI.pref_mem_usage = g_hash_table_new((GHashFunc)g_int_hash, (GEqualFunc)g_int_equal);
     GAI.has_preferences = GAI_PREF_NONE;
 
-    GAI.menu_help_text = g_strdup ("Sorry, no help available.\n");
+    GAI.menu_help_text = g_strdup (_("Sorry, no help available.\n"));
 
-    GAI.help_text = g_strdup ("Sorry, this new applet has no help"
+    GAI.help_text = g_strdup (_("Sorry, this new applet has no help"
 			      " how to use its preference window.\n"
-			      "I can only recommend trial and error.\n");
+			      "I can only recommend trial and error.\n"));
     GAI.tooltips = NULL;
     GAI.tooltips_msg = NULL;
     GAI.about = NULL;
@@ -447,15 +507,16 @@ gai_init_instance (GaiApplet *applet, const char *name, const char *version,  co
     GAI.foreground_alpha = 0;
 
 
-    GAI.menu_name = g_ptr_array_new();
-    GAI.menu_icon = g_ptr_array_new();
-    GAI.menu_type = g_array_new(FALSE, TRUE, sizeof(int));
-    GAI.menu_func = g_ptr_array_new();
-    GAI.menu_ptr = g_ptr_array_new();
+    GAI.menu_hash = g_hash_table_new_full(g_str_hash, g_str_equal, 
+					  (GDestroyNotify)menu_hash_free_key, 
+					  (GDestroyNotify)menu_hash_free_data);
+    GAI.menu_list = NULL;
     GAI.menu_entries = 0;
+    GAI.menu_changed = FALSE;
 
 
-#  ifdef GAI_WITH_GNOME
+
+#   ifdef GAI_WITH_GNOME
     if(name !=NULL)
 	gnome_program_init (name, version, LIBGNOMEUI_MODULE,
 			    (*argc), (*argv), 
@@ -466,9 +527,13 @@ gai_init_instance (GaiApplet *applet, const char *name, const char *version,  co
 			    (*argc), (*argv),
 			    GNOME_CLIENT_PARAM_SM_CONNECT, FALSE, 
 			    GNOME_PARAM_NONE);
-#else
+#   else
     gtk_init(argc,argv);
-#endif
+#   endif
+
+
+
+
 
 }
 
@@ -539,7 +604,7 @@ static void show_help(void)
 {
 
 	    printf(
-		"\nGAI library v"VERSION" command line options:\n"
+		"\nGAI library v%s command line options:\n"
 		"\t--gai-debug-on\t\tStores debug info in /tmp/gai-debug-output\n"
 		"\t\t\t\tabout all applets.\n"
 		"\t--gai-debug-on-this\tStores debug info about this applet only.\n"
@@ -565,7 +630,8 @@ static void show_help(void)
 #		endif		
 		"\t--gai-list-settings\tLists current global settings.\n"
 		"\t--gai-list-settings-this Lists settings for this applet.\n"
-		"\t--gai-help\t\tShows this text. (Strange, isn't it? ;-)\n\n");
+		"\t--gai-help\t\tShows this text. (Strange, isn't it? ;-)\n\n",
+		VERSION);
 }
 
 #define GAI_LIST_SETTINGS 0
@@ -678,68 +744,68 @@ gai_init_arguments (const char* name, int argc, char** argv)
 	    break;
 	case GAI_DEBUG_ON:
 	    set_all("gai/debug", "gai/%s-debug",1);
-	    printf("** GAI: Debug mode ON for ALL applets."
-		   " The output will be in /tmp/gai-debug-output\n");
+	    printf(_("** GAI: Debug mode ON for ALL applets."
+		   " The output will be in /tmp/gai-debug-output\n"));
 	    exit(0);
 	    break;
 	case GAI_DEBUG_ON_THIS:
 	    set_one("gai/%s-debug",(char *)name,1);
-	    printf("** GAI: Debug mode ON for THIS applets."
-		   " The output will be in /tmp/gai-debug-output\n");
+	    printf(_("** GAI: Debug mode ON for THIS applets."
+		   " The output will be in /tmp/gai-debug-output\n"));
 	    exit(0);
 
 	    break;
 	case GAI_DEBUG_OFF:
 	    set_all("gai/debug", "gai/%s-debug",0);
-	    printf("** GAI: Debug mode OFF for ALL applets.\n");
+	    printf(_("** GAI: Debug mode OFF for ALL applets.\n"));
 	    exit(0);
 
 	    break;
 	case GAI_DEBUG_OFF_THIS:
 	    set_one("gai/%s-debug",(char *)name,0);
-	    printf("** GAI: Debug mode OFF for THIS applets.\n");
+	    printf(_("** GAI: Debug mode OFF for THIS applets.\n"));
 	    exit(0);
 
 
 	    break;
 	case GAI_BROKEN_WM:
 	    set_all("gai/broken-wm", "gai/%s-broken-wm",1);
-	    printf("** GAI: Setting broken Window Manager mode for ALL applets.\n"
-		   "To turn off, use the switch --gai-working-wm\n");
+	    printf(_("** GAI: Setting broken Window Manager mode for ALL applets.\n"
+		   "To turn off, use the switch --gai-working-wm\n"));
 	    exit(0);
 
 	    break;
 	case GAI_BROKEN_WM_THIS:
 	    set_one("gai/%s-broken-wm",(char *)name,0);
-	    printf("** GAI: Setting broken Window Manager mode for THIS applet.\n"
-		   "To turn off, use the switch --gai-working-wm-this\n");
+	    printf(_("** GAI: Setting broken Window Manager mode for THIS applet.\n"
+		   "To turn off, use the switch --gai-working-wm-this\n"));
 	    exit(0);
 	    break;
 	case GAI_WORKING_WM:
 	    set_all("gai/broken-wm", "gai/%s-broken-wm",0);
-	    printf("** GAI: Removing broken Window Mananger mode for ALL applets.\n"
-		   "To turn on, use the switch --gai-broken-wm\n");
+	    printf(_("** GAI: Removing broken Window Mananger mode for ALL applets.\n"
+		   "To turn on, use the switch --gai-broken-wm\n"));
 	    exit(0);
 
 	    break;
 	case GAI_WORKING_WM_THIS:
 	    set_one("gai/%s-broken-wm",(char *)name,0);
-	    printf("** GAI: Removing broken Window Mananger mode for THIS applets.\n"
-		   "To turn on, use the switch --gai-broken-wm-this\n");
+	    printf(_("** GAI: Removing broken Window Mananger mode for THIS applets.\n"
+		   "To turn on, use the switch --gai-broken-wm-this\n"));
 	    exit(0);
 
 	    break;
 	case GAI_SIZE:
 	    set_all("gai/size", "gai/%s-size", atoi(argv[i+1]));
-	    printf("** GAI: Changes the default Dockapp/wmapplet size to %d for all applets\n"
-		   "Original size is 64\n",atoi(argv[i+1]));
+	    printf(_("** GAI: Changes the default Dockapp/wmapplet size to %d for all applets\n"
+		   "Original size is 64\n"),atoi(argv[i+1]));
 	    exit(0);
 
 	    break;
 	case GAI_SIZE_THIS:
 	    set_one("gai/%s-size", (char *)name, atoi(argv[i+1]));
-	    printf("** GAI: Changes the default Dockapp/wmapplet size to %d for this applet\n"
-		   "Original size is 64\n", atoi(argv[i+1]));
+	    printf(_("** GAI: Changes the default Dockapp/wmapplet size to %d for this applet\n"
+		   "Original size is 64\n"), atoi(argv[i+1]));
 	    exit(0);
 
 	    break;
@@ -822,8 +888,7 @@ gai_init(const char *name, const char *version, const char *image_path,
     g_assert((*argc_p) >0);
     g_assert((*argv_p) !=NULL);
 
-
-    gai_init_instance(NULL,name, version, image_path, argc_p, argv_p);
+    gai_init_instance(NULL, name, version, image_path, argc_p, argv_p);
     gai_init_arguments(name, (*argc_p), (*argv_p));
     return 0;
 }
@@ -834,10 +899,7 @@ int gai_init2(GaiApplet *applet, int *argc_p, char ***argv_p)
     g_assert((*argc_p) >0);
     g_assert((*argv_p) !=NULL);
 
-    /* Start debug */
-
     gai_init_instance(applet, NULL, NULL, NULL, argc_p, argv_p);
-
     gai_init_arguments(applet->name, (*argc_p), (*argv_p));
     return GAI.applet_type;
 
@@ -885,8 +947,6 @@ GdkPixbuf *gai_load_image_at_size(const char *fname, int width, int height)
 
     name_buff = g_strdup_printf ("%s/%s", GAI.applet.image_path, fname);
 
-    printf("post gtk 22\n");
-
     image = gdk_pixbuf_new_from_file_at_size(name_buff, width,height, &imerr);
 
     g_free(name_buff);
@@ -909,8 +969,6 @@ GdkPixbuf *gai_load_image_at_size(const char *fname, int width, int height)
     GdkPixbuf *image, *image2;
     GAI_ENTER; gai_is_init();
 
-    printf("pre gtk 22\n");
-
     image = gai_load_image(fname);
     if(image == NULL)
 	return NULL;
@@ -942,8 +1000,8 @@ gai_signal_on_update_interval_change (int delay)
     g_assert(delay >0);
 
     if (! GAI.init_done)
-	gai_display_error_quit ("You can only change the updating interval"
-				" after the init stage!");
+	gai_display_error_quit (_("You can only change the updating interval"
+				" after the init stage!"));
     if (GAI.on_update_callback)
     {
 	if (GAI.timer)
@@ -1066,7 +1124,7 @@ gai_signal_on_mouse_button_click (GaiCallback2 *function, int button, gpointer u
 	GAI.on_mouse_click2_userdata = userdata;
 	break;
     default:
-	gai_display_error_quit("Only mouse button one and two can be connected!");
+	gai_display_error_quit(_("Only mouse button one and two can be connected!"));
     }
     GAI_LEAVE;
 }
@@ -1091,7 +1149,7 @@ gai_signal_on_mouse_button_release(GaiCallback2 *function, int button, gpointer
 	GAI.on_mouse_release2_userdata = userdata;
 	break;
     default:
-	gai_display_error_quit("Only mouse button one and two can be connected!");
+	gai_display_error_quit(_("Only mouse button one and two can be connected!"));
 	break;
     }
 
@@ -1161,39 +1219,160 @@ gai_menu_add_help_text(const char *help_text)
     GAI_LEAVE;
 }
 
-void
-gai_menu_add(const char *name, const char *icon, int type, void *func, void *ptr)
+static MenuHashItem *gai_menu_create_menuitem(const char *name, const char *icon, int type, void *func, void *ptr)
 {
-    GAI_ENTER; gai_is_init();
-    g_assert(!GAI.init_done);
+    MenuHashItem *hash_item;
+    char *key;
+
     g_assert(type==GAI_MENU_SEPARATOR || 
 	     type==GAI_MENU_STOCK || 
 	     type==GAI_MENU_NONE ||
 	     type==GAI_MENU_FILE);
 
-    GAI.menu_entries++;
+    key = g_strdup_printf("%d", GAI.menu_entries);
+ 
+    hash_item = g_malloc0(sizeof(MenuHashItem));
 
     if(name!=NULL)
-	g_ptr_array_add(GAI.menu_name,(gpointer)g_strdup(name));
-    else
-	g_ptr_array_add(GAI.menu_name,NULL);
+	hash_item->name = g_strdup(name);
 
     if(icon!=NULL)
-	g_ptr_array_add(GAI.menu_icon,(gpointer)g_strdup(icon));   
-    else
-	g_ptr_array_add(GAI.menu_icon,NULL);
+	hash_item->icon = g_strdup(icon);   
+     
+    hash_item->type = type;
+    hash_item->func = func;
+    hash_item->ptr = ptr;
+    hash_item->key = key;
 
-    g_array_append_val(GAI.menu_type, type);
+    return hash_item;
+}
 
-    g_ptr_array_add(GAI.menu_func,(gpointer)func);
-    g_ptr_array_add(GAI.menu_ptr, (gpointer)ptr);
+static void gai_menu_update(void)
+{
+    GAI.menu_changed = TRUE;
 
+#ifdef GAI_WITH_GNOME
+    if(GAI.init_done && (GAI.applet_type == GAI_GNOME2 || GAI.applet_type == GAI_GNOME1))
+	gai_gnome_create_menu();
+#endif
+
+}
+
+int
+gai_menu_add(const char *name, const char *icon, int type, void *func, void *ptr)
+{
+    MenuHashItem *hash_item;
+
+    GAI_ENTER; gai_is_init();
+
+    hash_item = gai_menu_create_menuitem(name, icon, type, func, ptr);
+
+    g_hash_table_insert(GAI.menu_hash, hash_item->key, hash_item);
+
+    GAI.menu_list = g_slist_append(GAI.menu_list, hash_item->key);
+
+    GAI.menu_entries++;
+
+    gai_menu_update();
+
+    GAI_LEAVE;
+    return GAI.menu_entries;
+}
+
+void gai_menu_remove(int id)
+{
+
+    char *key;
+    MenuHashItem *hash_item;
+
+    GAI_ENTER; gai_is_init();
+
+    g_assert(id <= GAI.menu_entries);
+
+    key = g_strdup_printf("%d", id);
+
+    hash_item = g_hash_table_lookup(GAI.menu_hash, key);
+
+    if(hash_item != NULL){
+	GAI.menu_list = g_slist_remove(GAI.menu_list, hash_item->key);
+	g_hash_table_remove(GAI.menu_hash, hash_item->key);
+	/* key and hash_item gets free'd by g_hash_table remove */
+	gai_menu_update();
+    }
+    g_free(key);
     GAI_LEAVE;
 }
 
+int gai_menu_insert(int num, const char *name, const char *icon, int type, void *func, void *ptr)
+{
+    MenuHashItem *hash_item;
+    GAI_ENTER; gai_is_init();
+
+    hash_item = gai_menu_create_menuitem(name, icon, type, func, ptr);
+
+    GAI.menu_list = g_slist_insert(GAI.menu_list, hash_item->key, num);
+
+    GAI.menu_entries++;
+
+    gai_menu_update();
+
+    GAI_LEAVE;
+    return GAI.menu_entries;
+
+
+    
+}
+
+gboolean gai_menu_change(int id, const char *name, const char *icon, int type, void *func, void *ptr)
+{
+
+    char *key;
+    MenuHashItem *hash_item;
+
+    GAI_ENTER; gai_is_init();
+
+    g_assert(type==GAI_MENU_SEPARATOR || 
+	     type==GAI_MENU_STOCK || 
+	     type==GAI_MENU_NONE ||
+	     type==GAI_MENU_FILE);
+
+
+    key = g_strdup_printf("%d", id);
+
+    hash_item = g_hash_table_lookup(GAI.menu_hash, key);
+
+    g_free(key);
+
+    if(hash_item == NULL){
+	GAI_LEAVE;
+	return FALSE;
+    }
+
+    if(hash_item->name != NULL)
+	g_free(hash_item->name);
+
+    if(hash_item->icon != NULL)
+	g_free(hash_item->icon);
+
+    if(name!=NULL)
+	hash_item->name = g_strdup(name);
+
+    if(icon!=NULL)
+	hash_item->icon = g_strdup(icon);   
+     
+    hash_item->type = type;
+    hash_item->func = func;
+    hash_item->ptr = ptr;
+
+    gai_menu_update();
+
+    GAI_LEAVE;
+    return TRUE;
+
+}
 
 void 
-gai_signal_on_preferences(GaiCallback0 *function, gpointer userdata)
+gai_signal_on_preferences(GaiCallback1 *function, gpointer userdata)
 {
 
     GAI_ENTER;  gai_is_init();
@@ -1207,7 +1386,7 @@ gai_signal_on_preferences(GaiCallback0 *function, gpointer userdata)
 
 void 
 gai_preferences(const char *name, GaiNoteBook *gn, const char *help_text, 
-		GaiCallback0 *function, gpointer userdata)
+		GaiCallback1 *function, gpointer userdata)
 {
     GAI_ENTER;  gai_is_init();
 
@@ -1234,7 +1413,7 @@ gai_preferences(const char *name, GaiNoteBook *gn, const char *help_text,
 
 
 void gai_preferences2(const char *name, GaiPI *pref_instr, const char *help_text, 
-		GaiCallback0 *function, gpointer userdata)
+		GaiCallback1 *function, gpointer userdata)
 {
     GAI_ENTER;  gai_is_init();
 
@@ -1339,10 +1518,10 @@ gai_flags_set(GaiFlagsType gf)
 #ifdef GAI_WITH_GL
 	GAI.open_gl = 1;
 #else
-	gai_display_error_quit("You're trying to run an applet that uses OpenGL.\n"
+	gai_display_error_quit(_("You're trying to run an applet that uses OpenGL.\n"
 			       "You have not compiled with OpenGL support in GAI.\n"
 			       "Please visit http://gtkglext.sf.net for GtkGlExt\n"
-			       "which is required for OpenGL support.\n");
+			       "which is required for OpenGL support.\n"));
 #endif
     }
     GAI_LEAVE;
